d("customPickupPersonView", ['ccrz', 'util', 'customPickupPersonModel'], function (CCRZ, UTIL, customPickupPersonModel) {

    return {
            extendcustomPickupPersonView: function () {
                CCRZ.custom = _.extend(CCRZ.custom || {});
                CCRZ.custom.views = _.extend(CCRZ.custom.views || {});
                CCRZ.custom.models = _.extend(CCRZ.custom.models || {});
                CCRZ.remoteActions = _.extend(CCRZ.remoteActions || {});
    
                CCRZ.views.customPickupPersonView = CCRZ.CloudCrazeView.extend({
                    el: CCRZ.uiProperties.customPickupPersonView.desktop.selector,
                    viewName: "customPickupPersonView",
                    subscriptionTemplate: CCRZ.util.template(CCRZ.uiProperties.customPickupPersonView.desktop.tmpl),
                    init: function () {
                        UTIL.showLoader();
                        this.listenTo(this.model, "reset", this.ready);
                        this.dataLoadStarted = false;
                        this.dataReady = false;
                        this.dataModel = this.model;
                    },
                    initSetup: function(callback) {
                        //CCRZ-2335 : Different Tabs under MyAccount do not pull data real-time
                        //reset the dataLoadStarted and dataReady flags and refresh the view
                        this.dataLoadStarted = false;
                        this.dataReady = false;
    
                        callback();
                    },
                    preRender: function() {
                        UTIL.showLoader();
                        let v = this;
                        if(!v.dataLoadStarted) {
                            v.dataModel.fetch(function (res, event) {
                                if (res && res.data) {
                                    v.model.attributes.pickupPersons = res.data;
                                    v.dataReady = true; 
                                    v.render();
                                }
                            });
                            v.dataLoadStarted=true;
                        }
                    },
                    renderDesktop: function () {
                        if(this.dataReady) {
                            this.renderData();
                        }
                    },
                    renderPhone: function() {
                        if(this.dataReady) {
                            this.renderData();
                        }
                    },
                    renderData: function() {
                        let _this = this;
    
                        _this.$el.html(this.subscriptionTemplate(this.model.toJSON()));
                        _this.initValidation();
                        setTimeout(function () {
                            UTIL.hideLoader();
                        }, 100);
                    },
                    events: {
                        "click .addPerson" : "handleNewPerson",
                        "click .editPerson" : "handleEditPerson",
                        "click .deletePerson" : "handleDeletePerson",
                        "click .cancelPerson" : "handleCancel",
                        "click .savePerson" : "handleSave"
                    },
                    initValidation: function() {
                        $('#editPersonForm').validate({
                            invalidHandler: function(event1, validator1) {
                                CCRZ.handleValidationErrors(event1, validator1, 'personModal-messagingSection-Error', false);
                            },
                            rules: {
                                "personBean.firstName": {
                                    required: true
                                },
                                "personBean.lastName": {
                                    required: true
                                },
                                "personBean.email": {
                                    required: true,
                                    email: true
                                },
                                "personBean.phoneNumber": {
                                    minlength: 10,
                                    maxlength: 10,
                                    required: true
                                }
                            },
                            messages: {
                                "personBean.firstName": {
                                 required: "First Name is required."
                                },
                                "personBean.lastName": {
                                    required: "Last Name is required."
                                },
                                "personBean.email": {
                                    required: "Email is required.",
                                    email: "Your email address must be in the format of an email address"
                                },
                                "personBean.phoneNumber": {
                                    required: "Phone Number is required."
                                }
                            },
                            errorPlacement: function(error, element) { }
                        });
                    },
                    handleNewPerson: function() {
                        let newBean = {
                            sfid: ''
                        };
                        this.openPersonModal(newBean);
                    },
                    handleEditPerson: function() {
                        let objLink = $(event.target);
                        let id = objLink.data("id");
                        let pickupPersons = this.model.attributes.pickupPersons;
                        let person = _.findWhere(pickupPersons, {sfid: id});
                        this.openPersonModal(person);
                    },
                    openPersonModal: function(personObj){
                        this.clearPersonModal();
                        if(personObj.sfid == '') {
                            $('.editPickupPerson').modal("show");
                        } else {
                            let form = $('#editPersonForm');
                            $(form).find('.cc_firstName').val(personObj.firstName);
                            $(form).find('.cc_lastName').val(personObj.lastName);
                            $(form).find('.cc_email').val(personObj.email);
                            $(form).find('.cc_phone').val(personObj.phone);
                            $(form).find('.personId').val(personObj.sfid);
                            $('.editPickupPerson').modal("show");
                        }
                    },
                    handleCancel: function() {
                        this.clearPersonModal();
                        this.closePersonModal();
                    },
                    clearPersonModal: function() {
                        let form = $('#editPersonForm');
                        $(form).find('.cc_firstName').val('');
                        $(form).find('.cc_lastName').val('');
                        $(form).find('.cc_email').val('');
                        $(form).find('.cc_phone').val('');
                        $(form).find('.cc_firstName').removeClass('error');
                        $(form).find('.cc_lastName').removeClass('error');
                        $(form).find('.cc_email').removeClass('error');
                        $(form).find('.cc_phone').removeClass('error');
                        $('.personModal-messagingSection-Error').html('');
                    },
                    closePersonModal:function() {
                        $('.editPickupPerson').modal("hide");
                    },
                    handleSave: function() {
                        UTIL.showLoader();
                        var isValid = false;
                        isValid = $("#editPersonForm").valid();
                        if(isValid){
                            var formData = form2js('editPersonForm', '.', false, function(node) {}, false, true);
                            var personData = formData.personBean;
                            var remoteCall = _.extend(CCRZ.RemoteInvocation, {className: 'DCMB2B_subscription_controller'});
                            remoteCall.invokeCtx(
                                'savePickupPersonExtension', (JSON.stringify(personData)),
                                function (res) {
                                    if (res && res.success) {
                                        
                                        let pickupPersonView = CCRZ.customPickupPersonView;
                                        pickupPersonView.closePersonModal();
                                        CCRZ.customPickupPersonView.dataModel.fetch(function (res, event) {
                                            if (res && res.data) {
                                                pickupPersonView.model.attributes.pickupPersons = res.data;
                                                pickupPersonView.dataReady = true; 
                                                pickupPersonView.render();
                                            }
                                        });
                                        UTIL.hideLoader();
                                    } else {
                                        // Error handling
                                        CCRZ.pubSub.trigger("pageMessage", CCRZ.createPageMessage('ERROR', "personModal-messagingSection-Error", 'Error Adding/Editing Pickup Person'));
                                        UTIL.hideLoader();
                                    }
                                }, {buffer: false, escape: false, nmsp: false}
                            );
                        }
                    },
                    handleDeletePerson: function() {
                        UTIL.showLoader();
                        let objLink = $(event.target);
                        let pickUpPersonId = objLink.data("id");
                        var remoteCall = _.extend(CCRZ.RemoteInvocation, {className: 'DCMB2B_subscription_controller'});
                        remoteCall.invokeCtx(
                            'deletePickupPersonExtension', pickUpPersonId,
                            function (res) {
                                if (res && res.success) {
                                    let pickupPersonView = CCRZ.customPickupPersonView;
                                    pickupPersonView.dataModel.fetch(function (res, event) {
                                        if (res && res.data) {
                                            pickupPersonView.model.attributes.pickupPersons = res.data;
                                            pickupPersonView.dataReady = true; 
                                            pickupPersonView.render();
                                        }
                                    });
                                    UTIL.hideLoader();
                                } else {
                                    // Error handling
                                    CCRZ.pubSub.trigger("pageMessage", CCRZ.createPageMessage('ERROR', "personModal-messagingSection-Error", 'Error Adding/Editing Pickup Person'));
                                    UTIL.hideLoader();
                                }
                            }, {buffer: false, escape: false, nmsp: false}
                        );
                    }
                });
    
                CCRZ.customPickupPersonView = new CCRZ.views.customPickupPersonView({
                    model: new customPickupPersonModel()
                });
                return CCRZ.customPickupPersonView;
            }
    
        }
    });